/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, console*/

/**
 * Apps module.
 */
define({
    name: 'models/apps',
    requires: ['core/tizen'],
    def: function apps(tizen) {
        'use strict';

        /**
         * Obtains apps currently running on device and passes them to the
         * specified callback function.
         * @param {ApplicationInformation[]} allApps All apps installed on
         * device.
         * @param {function} onSuccess
         */
        function getLaunchedApps(allApps, onSuccess) {
            try {
                tizen.application.getAppsContext(
                    function onGetAppsContext(launchedAppsContexts) {
                        var launchedApps = [],
                            appsLength = allApps.length,
                            contextsLength = launchedAppsContexts.length,
                            i = 0,
                            j = 0,
                            app = null,
                            appContext = null;

                        for (i = 0; i < appsLength; i += 1) {
                            app = allApps[i];
                            for (j = 0; j < contextsLength; j += 1) {
                                appContext = launchedAppsContexts[j];
                                if (appContext.appId === app.id &&
                                    launchedApps.indexOf(app) === -1) {
                                    launchedApps.push(app);
                                }
                            }
                        }
                        onSuccess(launchedApps);
                    });
            } catch (error) {
                console.error(error);
            }
        }

        /**
         * Obtains apps installed on device and pass them to the specified
         * callback function.
         * @param {string} filter If equals 'all' all apps will be passed.
         * If equals 'launched' only currently running apps will be passed.
         * @param {function} onSuccess
         * @param {function} onError
         */
        function getApps(filter, onSuccess, onError) {
            try {
                tizen.application.getAppsInfo(function onGetApps(apps) {
                    if (filter === 'all') {
                        onSuccess(apps);
                    } else if (filter === 'launched') {
                        getLaunchedApps(apps, onSuccess);
                    } else {
                        onError(new Error('Wrong filter argument'));
                    }
                }, onError);
            } catch (error) {
                console.error(error);
            }
        }

        /**
         * Returns an ApplicationInformation object of the application with
         * the specified id.
         * @param {string} appId application id.
         * @returns {ApplicationInformation}
         */
        function getApp(appId) {
            try {
                return tizen.application.getAppInfo(appId);
            } catch (error) {
                console.error(error);
            }
        }

        /**
         * Launches an application with the given application ID.
         * @param {string} appId
         * @param {function} onSuccess
         * @param {function} onError
         */
        function launch(appId, onSuccess, onError) {
            try {
                tizen.application.launch(appId, onSuccess, onError);
            } catch (error) {
                console.error(error);
            }
        }

        return {
            getApps: getApps,
            getApp: getApp,
            launch: launch
        };
    }
});
