/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, tau, document, console*/

/**
 * Main view module.
 */
define({
    name: 'views/main',
    requires: [
        'core/application',
        'core/template',
        'core/event',
        'helpers/uimenu',
        'models/apps'
    ],
    def: function main(req) {
        'use strict';

        /**
         * The core application module.
         * @type {Module}
         */
        var application = req.core.application,

            /**
             * The core template module.
             * @type {Module}
             */
            template = req.core.template,

            /**
             * The core event module.
             * @type {Module}
             */
            event = req.core.event,

            /**
             * The ui menu helper module.
             * @type {Module}
             */
            uiMenu = req.helpers.uimenu,

            /**
             * The apps model module.
             * @type {Module}
             */
            apps = req.models.apps,

            /**
             * Reference to the list of applications.
             * @type {HTMLElement}
             */
            appsListView = null,

            /**
             * Applications filter.
             * @type {string}
             */
            appsFilter = 'all',

            /**
             * Ui menu handler.
             * @type {object}
             */
            menu = null,

            /**
             * Reference to the page element.
             * @type {HTMLElement}
             */
            page = null;

        /**
         * Updates the list of applications.
         */
        function updateAppsList() {
            apps.getApps(appsFilter, function onGetApps(appsList) {
                var app = null,
                    length = appsList.length,
                    i = 0,
                    html = '';

                for (i = 0; i < length; i += 1) {
                    app = appsList[i];
                    if (app.name) {
                        html += template.get('app-row', app);
                    }
                }
                appsListView.innerHTML = html;
            }, function onError(e) {
                appsListView.innerHTML = '';
                console.error(e.message);
            });
        }

        /**
         * Handles the 'click' event on the 'all' menu item.
         */
        function onAllAppsClick() {
            appsFilter = 'all';
            updateAppsList();
        }

        /**
         * Handles the 'click' event on the 'launched' menu item.
         */
        function onLaunchedAppsClick() {
            appsFilter = 'launched';
            updateAppsList();
        }

        /**
         * Binds hardware keys events.
         */
        function bindTizenHwKeyEvents() {
            document.addEventListener('tizenhwkey', function onTizenhwkey(e) {
                var popupNotActive = !document
                        .querySelector('.ui-popup-active');

                if (e.keyName === 'back' &&
                    tau.activePage.id === 'main' && popupNotActive) {
                    application.exit();
                }
            });
        }

        /**
         * Binds click events on menu items.
         */
        function bindMenuButtonsEvents() {
            menu.bindClickEvent('all-apps', onAllAppsClick);
            menu.bindClickEvent('launched-apps', onLaunchedAppsClick);
        }

        /**
         * Binds click events on the applications list.
         */
        function bindListEvents() {
            appsListView.addEventListener('click', function onClick(e) {
                event.fire('clicked', e.target.dataset.id);
            });
        }

        function bindPageEvents() {
            document.addEventListener('visibilitychange', updateAppsList);
            page.addEventListener('pageshow', updateAppsList);
        }

        /**
         * Binds events.
         */
        function bindEvents() {
            bindTizenHwKeyEvents();
            bindMenuButtonsEvents();
            bindListEvents();
            bindPageEvents();
        }

        /**
         * Initializes the module.
         */
        function init() {
            page = document.getElementById('main');
            appsListView = document.getElementById('apps-list-view');
            menu = uiMenu.createUiMenu('main', 'main-menu-popup');

            bindEvents();
            updateAppsList();
        }

        return {
            init: init
        };
    }
});
